"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const fs_1 = require("fs");
const path_1 = require("path");
const errors_1 = require("../../errors");
const utils_1 = require("../../utils");
const types_1 = require("../../types");
const toolingApi_1 = require("../toolingApi");
const baseDeploy_1 = require("./baseDeploy");
const index_1 = require("./index");
class AuraDeploy extends baseDeploy_1.BaseDeploy {
    deploy(component) {
        return __awaiter(this, void 0, void 0, function* () {
            this.component = component;
            const auraDefinitions = yield this.buildDefList();
            try {
                const promiseArray = auraDefinitions.map((def) => __awaiter(this, void 0, void 0, function* () { return this.upsert(def); }));
                const results = yield Promise.all(promiseArray);
                return this.formatBundleOutput(results);
            }
            catch (e) {
                const failures = [
                    this.createDeployResult(auraDefinitions[0].FilePath, false, false, e.message)
                ];
                return this.formatBundleOutput(failures, true);
            }
        });
    }
    buildDefList() {
        return __awaiter(this, void 0, void 0, function* () {
            const sourceFiles = this.component.sources;
            const auraDefinitions = [];
            const existingDefinitions = yield this.findExistingDefinitions();
            let bundleId;
            if (existingDefinitions.length > 0) {
                bundleId = existingDefinitions[0].AuraDefinitionBundleId;
            }
            else {
                const newBundle = yield this.createBundle();
                bundleId = newBundle.id;
            }
            sourceFiles.forEach((sourceFile) => __awaiter(this, void 0, void 0, function* () {
                const source = fs_1.readFileSync(sourceFile, 'utf8');
                const suffix = utils_1.extName(sourceFile);
                const defType = this.getAuraDefType(sourceFile, suffix);
                const format = this.getAuraFormat(suffix);
                let match;
                if (existingDefinitions.length > 0) {
                    match = existingDefinitions.find(definition => definition.DefType === defType);
                }
                const auraDef = Object.assign({ FilePath: sourceFile, DefType: defType, Source: source, Format: format }, (match ? { Id: match.Id } : { AuraDefinitionBundleId: bundleId }));
                // This is to ensure we return the correct project path when reporting errors
                // must be the file associated with the specified aura type
                index_1.AURA_TYPES.includes(auraDef.DefType)
                    ? auraDefinitions.unshift(auraDef)
                    : auraDefinitions.push(auraDef);
            }));
            return auraDefinitions;
        });
    }
    createBundle() {
        return __awaiter(this, void 0, void 0, function* () {
            const metadataContent = fs_1.readFileSync(this.component.xml, 'utf8');
            const metadataField = this.buildMetadataField(metadataContent);
            const bundleObject = {
                FullName: this.component.fullName,
                Metadata: metadataField
            };
            const newBundle = yield this.toolingCreate(this.component.type.name, bundleObject);
            if (!newBundle.success) {
                throw new errors_1.DeployError('error_creating_metadata_type', this.component.type.name);
            }
            return newBundle;
        });
    }
    upsert(auraDef) {
        return __awaiter(this, void 0, void 0, function* () {
            const type = this.component.type.name;
            if (auraDef.Id) {
                const formattedDef = {
                    Source: auraDef.Source,
                    Id: auraDef.Id
                };
                yield this.connection.tooling.update(toolingApi_1.deployTypes.get(type), formattedDef);
                return this.createDeployResult(auraDef.FilePath, true, false);
            }
            else {
                const formattedDef = {
                    AuraDefinitionBundleId: auraDef.AuraDefinitionBundleId,
                    DefType: auraDef.DefType,
                    Format: auraDef.Format,
                    Source: auraDef.Source
                };
                yield this.toolingCreate(toolingApi_1.deployTypes.get(type), formattedDef);
                return this.createDeployResult(auraDef.FilePath, true, true);
            }
        });
    }
    formatBundleOutput(deployResults, failure) {
        let toolingDeployResult;
        if (failure) {
            toolingDeployResult = {
                State: types_1.DeployStatusEnum.Failed,
                ErrorMsg: deployResults[0].problem,
                DeployDetails: {
                    componentSuccesses: [],
                    componentFailures: deployResults
                },
                isDeleted: false,
                metadataFile: this.component.xml
            };
        }
        else {
            toolingDeployResult = {
                State: types_1.DeployStatusEnum.Completed,
                DeployDetails: {
                    componentSuccesses: deployResults,
                    componentFailures: []
                },
                isDeleted: false,
                outboundFiles: this.component.sources,
                ErrorMsg: null,
                metadataFile: this.component.xml
            };
        }
        return toolingDeployResult;
    }
    getAuraFormat(suffix) {
        switch (suffix) {
            case 'js':
                return 'JS';
            case 'css':
                return 'CSS';
            case 'svg':
                return 'SVG';
            default:
                return 'XML';
        }
    }
    getAuraDefType(sourcePath, suffix) {
        const fileName = utils_1.baseName(sourcePath);
        switch (suffix) {
            case 'app':
                return 'APPLICATION';
            case 'cmp':
                return 'COMPONENT';
            case 'auradoc':
                return 'DOCUMENTATION';
            case 'css':
                return 'STYLE';
            case 'evt':
                return 'EVENT';
            case 'design':
                return 'DESIGN';
            case 'svg':
                return 'SVG';
            case 'js':
                if (fileName.endsWith('Controller')) {
                    return 'CONTROLLER';
                }
                else if (fileName.endsWith('Helper')) {
                    return 'HELPER';
                }
                else if (fileName.endsWith('Renderer')) {
                    return 'RENDERER';
                }
                break;
            case 'tokens':
                return 'TOKENS';
            case 'intf':
                return 'INTERFACE';
            default:
                return '';
        }
    }
    findExistingDefinitions() {
        return __awaiter(this, void 0, void 0, function* () {
            const auraDefResult = yield this.connection.tooling.query(`Select AuraDefinitionBundleId, Id, Format, Source, DefType from AuraDefinition where AuraDefinitionBundle.DeveloperName = '${this.component.fullName}'`);
            return auraDefResult.records;
        });
    }
    createDeployResult(filepath, success, created, problem) {
        const formattedPaths = this.getFormattedPaths(filepath);
        const result = {
            success,
            deleted: false,
            fileName: formattedPaths[0],
            fullName: formattedPaths[1],
            componentType: this.component.type.name
        };
        if (success) {
            result['created'] = created;
            result['changed'] = !created;
        }
        else {
            result['problem'] = problem;
            result['changed'] = false;
            result['created'] = false;
        }
        return result;
    }
    getFormattedPaths(filepath) {
        const pathParts = filepath.split(path_1.sep);
        const typeFolderIndex = pathParts.findIndex(part => part === this.component.type.directoryName);
        return [
            pathParts.slice(typeFolderIndex).join(path_1.sep),
            pathParts.slice(typeFolderIndex + 1).join(path_1.sep)
        ];
    }
}
exports.AuraDeploy = AuraDeploy;
//# sourceMappingURL=auraDeploy.js.map